globals [
  n-knowledge-fields                               ; number of different knowledge fields
  max-mc-length                                    ; max. length of marketing concept
  M                                                ; message of supplier
  joint-M                                          ; joint message of supplier and manufacturer
  EM                                               ; expected message of applicator 
  SCV                                              ; offered customer value (supplier, non-cooperative VCM)
  SMCV                                             ; offered customer value (supplier + manufacturer, cooperative VCM)
  ECV                                              ; expected customer value (applicator, cooperative and non-cooperative VCM)
    
  s-customer-value-tolerance                       ; (depend on the knowledge overlap between supplier and applicator), not used at the moment  
  s-m-customer-value-tolerance                     ; (depend on the knowledge overlap between supplier + manufacturer and applicator), not used at the moment 
  randomsupplier                                   ; identify one of the suppliers
  randomapplicator                                 ; identity one of the applicators
  randommanufacturer                               ; identify one of the manufacturers
  
  nc-costfactor                                    ; non-cooperative cost factor, not used at the moment
  c-costfactor                                     ; cooperative cost factor, not used at the moment
  cost                                         
  nc-knowledge-overlap                             ; non-cooperative knowledge overlap, not used at the moment
  c-knowledge-overlap                              ; cooperative knowledge overlap, not used at the moment
  setup-reminder
                                
  implementation-list                              ; list of of all implementation attemps (0 = failure; 1 = success)
  applicator-implementation-list                   ; list of all applicators
  supplier-implementation-list                     ; list of all suppliers
    
  ;strategy                                        ; set with interface chooser
  ;innovation-rate                                 ; set with interface slider (number of active suppliers)
  ;number-of-suppliers                             ; set with interface slider
  ;big-supplier's-ratio                            ; set with interface slider
  ;number-of-manufacturers                         ; set with interface slider
  ;number-of-applicators                           ; set with interface slider
]


breed [suppliers supplier]
breed [manufacturers manufacturer]            
breed [applicators applicator]

suppliers-own [
  ;kene
  sknowledge-fields                                ; the supplier's kene part 1
  samounts                                         ; the supplier's kene part 2
  sexpertises                                      ; the supplier's kene part 3
  smc                                              ; the suppplier's marketing concept
  succeed?                                         ; is supplier successful?
  further-improvement?                             ; is there any further improvement?
  implemented?                                     ; is innovation implemented (for print out)
  
  ;supplier
  capital                                          ; the amount of capital of the firm    
  supplier-applicator-memory                       ; applicator's ID of chosen applicator
  supplier-manufacturer-memory                     ; manufacturer's ID of chosen manufacturer
  applicator-preference                            ; applicator's ID for a successful implementation
  manufacturer-preference                          ; manufacturer's ID for a successful implementation
  active?                                          ; is chosen supplier active?
  suppliertick                                     ; tick of single supplier
  costs                                            ; cost for each step 
  sumcosts                                         ; total costs
  acceptance?                                      ; is supplier accepted?
  attributes                                       ; attributes for print-out
  strategy-choice
  ]

manufacturers-own [
  ;kene 
  mknowledge-fields                                ; the manufacturer's kene part 1
  mamounts                                         ; the manufacturer's kene part 2
  mexpertises                                      ; the manufacturer's kene part 3
  ]
                         
applicators-own [
  ;kene
  aknowledge-fields                                ; the applicator's kene part 1
  aamounts                                         ; the applicator's kene part 2
  aexpertises                                      ; the applicator's kene part 3
  amc                                              ; the applicator's marketing concept
  
  customer-value-memory
  selection-progress                               ; applicator selects an innovation out of three offered  
  cooperative-manufacturer-link-memory             ; manufacturer's ID, if strategy is cooperative
  cooperative-supplier-link-memory                 ; supplier's ID, if strategy is cooperative
  non-cooperative-supplier-link-memory             ; supplier's ID, if strategy is non-cooperative
  past-supplier-strategy                           ; memory of former supplier's strategy
  ]
         

to setup
  ifelse trials-number > 0 
  [
   no-display
   __clear-all-and-reset-ticks 
   ask patches [set pcolor white]                  ; not used at the moment
   set max-mc-length 2                             ; length of marketing concept = 2 triples
   set n-knowledge-fields 6                        ; number of knowledge fields = 6
  
   set implementation-list []
   set applicator-implementation-list []
   set supplier-implementation-list []
  
   initialise-manufacturers               
   ask manufacturers [setup-manufacturer]
  
   initialise-applicators             
   ask applicators [setup-applicator]
  
   initialise-supplier  
   ask suppliers [setup-supplier]

   activate-suppliers
   choose-strategy
   setup-output
   set setup-reminder true
  ]  
  [print "please insert trial-number"]
end


to initialise-supplier
  create-suppliers number-of-suppliers 
  [
   set capital initial-capital                     ; set in the input  
   set further-improvement? true                   ; set all initial values
   set succeed? false
   set active? false
   set suppliertick 0
   set costs 0
   set sumcosts 0
  ]   
  
  ask n-of round (big-supplier's-ratio * number-of-suppliers) suppliers        ; double the initial capital of several suppliers set in the interface tab  
  [set capital 2 * capital]                   
 
  ask suppliers 
  [  
   set smc []
   set sknowledge-fields []                        ; create a null kene
   set samounts []
   set sexpertises []
   set applicator-preference -1                    ; will be replaced by an ID Number >= 0 
   set manufacturer-preference -1                  ; will be replaced by an ID Number >= 0
  ]
end

to initialise-manufacturers 
  create-manufacturers number-of-manufacturers 
  ask manufacturers 
  [
   set mknowledge-fields []                        ; create a null kene
   set mamounts []
   set mexpertises []
  ]
end

to initialise-applicators
  create-applicators number-of-applicators 
  ask applicators 
  [ 
   set amc []
   set aknowledge-fields []                        ; create a null kene
   set aamounts []
   set aexpertises []
   set customer-value-memory 0
   set selection-progress 1
   set cooperative-manufacturer-link-memory -1     ; will be replaced by an ID Number >= 0
   set cooperative-supplier-link-memory -1         ; will be replaced by an ID Number >= 0
   set non-cooperative-supplier-link-memory -1     ; will be replaced by an ID Number >= 0
   set past-supplier-strategy "-"
  ]
end


to activate-suppliers
  let activesuppliernumber round (innovation-rate * number-of-suppliers)        ; activate the number of suppliers set in the interface tab         
  ask n-of activesuppliernumber suppliers [set active? true]
end


to choose-strategy
  let active-supplierlist [who] of suppliers with [active? = true]              ; choose random strategy for each active supplier
  let i 0 
  while [i < length active-supplierlist]
  [let a-s-number item i active-supplierlist
   ask supplier a-s-number [set strategy-choice random 2]
   set i i + 1]  
end  
  

to setup-supplier 
  make-skene
  make-smc
end
  
to setup-manufacturer
  make-mkene
end

to setup-applicator
  make-akene
  make-amc
end 


to make-skene
  let kene-capacity 8
  while [length sknowledge-fields < kene-capacity] 
  [
   if length sknowledge-fields <= 2 [set sknowledge-fields lput (random 2 + 1) sknowledge-fields]           ; random values 1-2
   if length sknowledge-fields = 3 [set sknowledge-fields lput (random 4 + 3) sknowledge-fields]            ; random values 3-6
   if length sknowledge-fields > 3 and length sknowledge-fields <= 5 
   [
    let a item 3 sknowledge-fields
    ifelse a <= 3
    [set sknowledge-fields lput (random 2 + 3) sknowledge-fields]                                           ; random values 3-4
    [set sknowledge-fields lput (random 2 + 5) sknowledge-fields]                                           ; random values 5-6
   ]                                  
   if length sknowledge-fields > 5 [set sknowledge-fields lput (random 3 + 4) sknowledge-fields]            ; random values 4-6    
  ]
 
  let i 0
  while [i < kene-capacity]
  [
   let knowledge item i  sknowledge-fields 
   ifelse knowledge <= 2 
   [set samounts lput (random 1 + 3) samounts]                                                              ; value 3
   [set samounts lput (random 3 + 1) samounts]                                                              ; random values 1-3
   set i i + 1
  ]
 
  while [length sexpertises < kene-capacity]
  [set sexpertises fput ((random 3) + 1) sexpertises]                                                       ; random values 1-3 
end


to make-mkene 
  let kene-capacity 8
  while [length mknowledge-fields < kene-capacity] 
  [
   if length mknowledge-fields <= 1 [set mknowledge-fields lput (random 2 + 1) mknowledge-fields]           ; random values 1-2
   if length mknowledge-fields > 1 and length mknowledge-fields <= 5 
   [set mknowledge-fields lput (random 2 + 3) mknowledge-fields]                                            ; random values 3-4       
   if length mknowledge-fields > 5 [set mknowledge-fields lput (random 2 + 5) mknowledge-fields]            ; random values 5-6    
  ]
 
  let i 0
  while [i < kene-capacity]
  [
   let knowledge item i mknowledge-fields
   ifelse knowledge >= 3 and knowledge <= 4
   [set mamounts lput (random 1 + 3) mamounts]                                                              ; value 3                                                           
   [set mamounts lput (random 3 + 1) mamounts]                                                              ; random values 1-3                                           
   set i i + 1
  ]
 
  while [length mexpertises < kene-capacity]
  [set mexpertises fput ((random 3) + 1) mexpertises]                                                       ; random values 1-3  
end               
 
 
to make-akene
  let kene-capacity 8
  while [length aknowledge-fields < kene-capacity] 
  [
   if length aknowledge-fields <= 4 [set aknowledge-fields lput (random 2 + 5) aknowledge-fields]           ; random values 5-6
   if length aknowledge-fields > 4 [set aknowledge-fields lput (random 4 + 1) aknowledge-fields]            ; random values 1-4                                        
  ] 
  
  let i 0
  while [i < kene-capacity]
  [
   let knowledge item i aknowledge-fields
   ifelse knowledge >= 5
   [set aamounts lput (random 1 + 3) aamounts]                                                              ; value 3                                                           
   [set aamounts lput (random 3 + 1) aamounts]                                                              ; random values 1-3                                               
   set i i + 1
  ]
 
  while [length aexpertises < kene-capacity]
  [set aexpertises fput ((random 3) + 1) aexpertises]                                                       ; random values 1-3
end
   
   
to make-smc                                                                                                 
  let numberlist []
  set smc []  
  let i 0
  while [ i < length sknowledge-fields] 
  [
   let number 0
   let knowledge item i sknowledge-fields
   foreach sknowledge-fields [if ? = knowledge and knowledge >= 3 [set number number + 1]]                  ; creates a list in which knowledge values >= 3 are described by their occurrence
   set numberlist lput number numberlist                                                                    ; knowledge values < 3 are described by putting a 0 to the list   
   set i i + 1   
  ]

  let sknowledge-fields-double sknowledge-fields
  let first-largest-occurrence max numberlist                                                               ; choose knowledge values with highest occurrence value and put its position within 
  let first-largest-occurrence-position position first-largest-occurrence numberlist                        ; the knowledge field to the marketing concept 
  set numberlist replace-item first-largest-occurrence-position numberlist 0
  set smc fput first-largest-occurrence-position smc
  let firstvalue item first-largest-occurrence-position sknowledge-fields 

  let numberlist2 []
  let i2 0
   while [ i2 < length sknowledge-fields] 
   [
    let knowledge2 item i2 sknowledge-fields
    foreach sknowledge-fields-double [if knowledge2 = firstvalue 
      [set sknowledge-fields-double replace-item i2 sknowledge-fields-double 0]] 
    set knowledge2 item i2 sknowledge-fields-double
    let number2 0
    foreach sknowledge-fields-double [if ? = knowledge2 and knowledge2 >= 3 [set number2 number2 + 1]]
    set numberlist2 lput number2 numberlist2
    set i2 i2 + 1   
   ]
  
  let second-largest-occurrence max numberlist2                                                              ; choose knowledge value with second highest occurrence value and put its 
  let second-largest-occurrence-position position second-largest-occurrence numberlist2                      ; position within the knowledge field to the marketing concept 
  let secondvalue item second-largest-occurrence-position sknowledge-fields
  ifelse secondvalue >= 3 
  [set smc fput second-largest-occurrence-position smc]
  [set sknowledge-fields-double sknowledge-fields
   set second-largest-occurrence max numberlist
   let second-alternative-occurrence-position position second-largest-occurrence numberlist
   set smc fput second-alternative-occurrence-position smc] 
   
  set smc sort smc  
end 


to make-amc
  let numberlist []
  set amc []  
  let i 0
  while [ i < length aknowledge-fields] 
  [
   let number 0
   let knowledge item i aknowledge-fields                                                                   ; create a list in which knowledge values >= 3 are described by their occurrence
   foreach aknowledge-fields [if ? = knowledge and knowledge >= 3 [set number number + 1]]                  ; knowledge values < 3 are described by putting a 0 to the list
   set numberlist lput number numberlist 
   set i i + 1   
  ]

  let aknowledge-fields-double aknowledge-fields
  let first-largest-occurrence max numberlist                                                               ; choose knowledge value with highest occurrence value and put its position within
  let first-largest-occurrence-position position first-largest-occurrence numberlist                        ; the knowledge field to the marketing concept
  set amc fput first-largest-occurrence-position amc
  let firstvalue item first-largest-occurrence-position aknowledge-fields 
  
  let numberlist2 []
  let i2 0
  while [ i2 < length aknowledge-fields] 
  [
   let knowledge2 item i2 aknowledge-fields
   foreach aknowledge-fields-double [if knowledge2 = firstvalue 
     [set aknowledge-fields-double replace-item i2 aknowledge-fields-double 0]]
   set knowledge2 item i2 aknowledge-fields-double
   let number2 0
   foreach aknowledge-fields-double [if ? = knowledge2 and knowledge2 >= 3 [set number2 number2 + 1]]
   set numberlist2 lput number2 numberlist2
   set i2 i2 + 1   
  ]
  
  let second-largest-occurrence max numberlist2                                                             ; choose knowledge value with second highest occurrence value and put its
  let second-largest-occurrence-position position second-largest-occurrence numberlist2                     ; position within the knowledge field to the marketing concept
  set amc fput second-largest-occurrence-position amc
  let secondvalue item second-largest-occurrence-position aknowledge-fields 
  
  set amc sort amc   
end 


to go
  if trials-number = 0 [__clear-all-and-reset-ticks stop]
  ifelse trials-number >= 0 and setup-reminder = true 
  [
   ifelse all? suppliers with [active? = true] 
   [capital = 0 or succeed? = true or further-improvement? = false] 
   [set trials-number trials-number - 1 ask suppliers [plotoutput] plotimplementation setup]
    [if count suppliers > 0 
      [   
       set randomsupplier one-of suppliers with [active? = true and capital > 0 and succeed? = false]
       while [randomsupplier = nobody] 
       [set randomsupplier one-of suppliers with [active? = true and capital > 0 and succeed? = false]]
     
       ask randomsupplier 
       [
        choose-applicator
        choose-manufacturer
       ]   
     ]
     pursue-vcm
    ]    
   tick
  ]
  [print "please reset first" stop]
end


to choose-applicator                                                            ; choose applicator
  ifelse applicator-preference != -1 
  [set randomapplicator applicator applicator-preference]
  [set randomapplicator one-of applicators]
end

to choose-manufacturer                                                          ; choose manufacturer
  set randommanufacturer one-of manufacturers
end


to pursue-vcm                                                                                               
  if randomsupplier != nobody 
  [ask randomsupplier 
    [set costs 0 
     ifelse further-improvement? = true and succeed? = false 
    [if strategy = "non-cooperative" [pursue-non-cooperative-vcm]               ; select strategy which can be set in the interface tab
     if strategy = "cooperative" [pursue-cooperative-vcm]
     if strategy = "random strategy"
     [if strategy-choice = 1 [pursue-non-cooperative-vcm]
      if strategy-choice = 0 [pursue-cooperative-vcm]]
    ]   
    [stop]
    ]
  ]  
end


to pursue-non-cooperative-vcm 
  let a item 0 smc
  let a0 item a sknowledge-fields
  let b item 1 smc
  let a1 item b sknowledge-fields
  let a3 0
  let a4 0
  let appllist []
 
  ask randomapplicator 
  [ 
   let d item 0 amc
   set a3 item d aknowledge-fields
   set appllist fput a3 appllist
   let f item 1 amc
   set a4 item f aknowledge-fields
   set appllist fput a4 appllist
  ]
  
  ifelse member? a0 appllist or member? a1 appllist                             ; comparison of knowledge fields of supplier and applicator
  [create-M
   ask randomapplicator [create-EM]
   pay-non-cooperative-marketing-cost-1 
   calculate-SCV 
   ask randomapplicator [calculate-ECV]
   compare-nc-customer-value]
  
  [set suppliertick suppliertick + 1
   set acceptance? 0
   pay-fixed-cost
   do-1st-non-cooperative-output
   adjust-expertises-of-supplier 
   start-next-attempt
   set applicator-preference -1
   set implementation-list lput 0 implementation-list]
end

to create-M
  let smc0k item 0 smc
  let k0 item smc0k sknowledge-fields
  let smc1k item 1 smc
  let k1 item smc1k sknowledge-fields
  let smc0a item 0 smc
  let a0 item smc0a samounts
  let smc1a item 1 smc
  let a1 item smc1a samounts
  set M (k0 * a0) + (k1 * a1)                                                   ; calculate supplier's message
end
 
to create-EM
  let amc0k item 0 amc
  let k0 item amc0k aknowledge-fields
  let amc1k item 1 amc
  let k1 item amc1k aknowledge-fields
  let amc0a item 0 amc
  let a0 item amc0a aamounts
  let amc1a item 1 amc
  let a1 item amc1a aamounts
  set EM (k0 * a0) + (k1 * a1)                                                  ; calculate expected message of applicator
end

to pay-non-cooperative-marketing-cost-1                                         ; calculate costs for the first non-cooperative attempt
  let difference abs (M - EM)
  set cost abs ((15 / 30) * difference + 1)
  set capital (capital - cost)
    
  set costs costs + cost
  set sumcosts sumcosts + cost 
   
  if capital < 2 
  [ask randomsupplier [set capital 0]]
end


to calculate-SCV                                                                                               
  let smc0k item 0 smc
  let k0 item smc0k sknowledge-fields
  let a0 item smc0k samounts
  let a0double a0
  ifelse a0 < 2 [set a0 0][set a0 1]                                                   
  let smc1k item 1 smc
  let k1 item smc1k sknowledge-fields
  let a1 item smc1k samounts
  let a1double a1
  ifelse a1 < 2 [set a1 0][set a1 1]
  
  let smc0e item 0 smc
  let e0 item smc0e sexpertises
  let smc1e item 1 smc
  let e1 item smc1e sexpertises 
  
  let elist []
  set elist lput e0 elist
  set elist lput e1 elist
  
  if k0 >= 3 and k0 <= 4 and k1 >= 5 and k1 <= 6 
  [if a0double = 3 [set k0 6]]
  
  
  if k1 >= 3 and k1 <= 4 and k0 >= 5 and k0 <= 6 
  [if a1double = 3 [set k1 6]]
  
  let list2 []
  set list2 lput k0 list2 
  set list2 lput k1 list2 
  
  set SCV (a0 *(k0 * e0)) + (a1 *(k1 * e1))                                     ; calculate supplier customer value
end

to calculate-ECV                                                                                                 
  let amc0k item 0 amc
  let k0 item amc0k aknowledge-fields
  let a0 item amc0k aamounts
  ifelse a0 < 2 [set a0 0][set a0 1]
  let amc1k item 1 amc
  let k1 item amc1k aknowledge-fields
  let a1 item amc1k aamounts
  ifelse a1 < 2 [set a1 0][set a1 1]
  let amc0e item 0 amc
  let e0 item amc0e aexpertises
  let amc1e item 1 amc
  let e1 item amc1e aexpertises 
  set ECV (a0 *(k0 * e0)) + (a1 *(k1 * e1))                                     ; calculate expected customer value of applicator
end

to compare-nc-customer-value                                                    ; compare non-cooperative customer value 
  let knowapplicator [] 
  let knowsupplier []
  set knowsupplier remove-duplicates sknowledge-fields
  set knowsupplier sort knowsupplier
  
  ask randomapplicator 
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  set nc-knowledge-overlap 0
  
  let i 0
  while                                                                         ; calculate non-cooperative knowledge overlap
  [i < length knowsupplier]
  [let svalue item i knowsupplier
   if member? svalue knowapplicator
   [set nc-knowledge-overlap nc-knowledge-overlap + 1]
   set i i + 1
  ]
 
  if nc-knowledge-overlap = 0 [set s-customer-value-tolerance 1.00]                                       ; not used at the moment
  if nc-knowledge-overlap > 0 and nc-knowledge-overlap <= 2 [set s-customer-value-tolerance 1.00]         ; not used at the moment
  if nc-knowledge-overlap > 2 and nc-knowledge-overlap <= 4 [set s-customer-value-tolerance 1.00]         ; not used at the moment
  if nc-knowledge-overlap > 4 [set s-customer-value-tolerance 1.00]                                       ; not used at the moment
  
  ifelse SCV * s-customer-value-tolerance >= ECV                                                             
  [ask randomapplicator
    [ifelse selection-progress <= 3 and SCV > customer-value-memory 
     [
      set customer-value-memory SCV
      set selection-progress selection-progress + 1
      
      if past-supplier-strategy = "cooperative"                                                            ; due to the higher customer value of current supplier, the applicator
      [ask supplier cooperative-supplier-link-memory [set succeed? false set implemented? 0]]              ; rejects the former supplier's attempt and changes related attributes 
       
       let j 0                         
       while [j < length implementation-list]
       [let value item j implementation-list
        let applicator item j applicator-implementation-list
        if value = 1 and applicator = [who] of randomapplicator 
        [set implementation-list replace-item j implementation-list 0]
        set j j + 1] 
   
       if past-supplier-strategy = "non-cooperative" 
       [ask supplier non-cooperative-supplier-link-memory [set succeed? false set implemented? 0]
       
        set j 0
        while [j < length implementation-list]
        [let value item j implementation-list
         let applicator item j applicator-implementation-list
         if value = 1 and applicator = [who] of randomapplicator 
         [set implementation-list replace-item j implementation-list 0]
         set j j + 1] 
       ]
      
       set non-cooperative-supplier-link-memory [who] of randomsupplier
       set past-supplier-strategy "non-cooperative"  
       
       ask randomsupplier 
       [set implemented? 1
        set suppliertick suppliertick + 1 
        set supplier-applicator-memory [who] of randomapplicator
        set applicator-preference [who] of randomapplicator
        pay-non-cooperative-marketing-cost-2
        set succeed? true 
        set acceptance? 1
        set implementation-list lput 1 implementation-list
        do-2nd-non-cooperative-output]
     ]
     
     [ask randomsupplier 
       [set suppliertick suppliertick + 1
        set implementation-list lput 0 implementation-list
        pay-non-cooperative-marketing-cost-2
        do-2nd-non-cooperative-output]
     ]  
    ]
  ]
   
  [set suppliertick suppliertick + 1
   set acceptance? 0
   pay-non-cooperative-marketing-cost-2
   set implementation-list lput 0 implementation-list
   do-2nd-non-cooperative-output
   set applicator-preference -1
   start-next-round]
end


to pay-non-cooperative-marketing-cost-2                                        ; calculate costs for second non-cooperative attempt
  let knowapplicator []
  let knowsupplier []
  set knowsupplier remove-duplicates sknowledge-fields
  set knowsupplier sort knowsupplier
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
   
  set nc-knowledge-overlap 0
  
  let i 0
  while 
  [i < length knowsupplier]
  [let svalue item i knowsupplier
   if member? svalue knowapplicator[set nc-knowledge-overlap nc-knowledge-overlap + 1]
   set i i + 1
  ]    

  if nc-knowledge-overlap = 0 [set nc-costfactor 1.00]                                               ; not used at the moment
  if nc-knowledge-overlap > 0 and nc-knowledge-overlap <= 2 [set nc-costfactor 1.00]                 ; not used at the moment
  if nc-knowledge-overlap > 2 and nc-knowledge-overlap <= 4 [set nc-costfactor 1.00]                 ; not used at the moment
  if nc-knowledge-overlap > 4 [set nc-costfactor 1.00]                                               ; not used at the moment
  
  let difference abs (SCV - ECV)
  set cost abs (((15 / 36) * difference + 1) * nc-costfactor)
  set capital (capital - cost)
    
  set costs costs + cost
  set sumcosts sumcosts + cost 
   
  if capital < 2 
  [ask randomsupplier [set capital 0]]
end


to pursue-cooperative-vcm
  let a item 0 smc
  let a0 item a sknowledge-fields
  let b item 1 smc
  let a1 item b sknowledge-fields
  let a3 0
  let a4 0
  let appllist []
  
  ask randomapplicator 
  [ 
   let d item 0 amc
   set a3 item d aknowledge-fields
   set appllist fput a3 appllist
   let f item 1 amc
   set a4 item f aknowledge-fields
   set appllist fput a4 appllist
  ]
  
  let identicalness 0                                                          ; comparison of knowledge fields
  ifelse a0 = a3 
  [set identicalness identicalness + 1 set a3 0]
  [if a0 = a4 [set identicalness identicalness + 1 set a4 0]]
  ifelse a1 = a3
  [set identicalness identicalness + 1 set a3 0]
  [if a1 = a4 [set identicalness identicalness + 1 set a4 0]]
  
  ifelse identicalness = 2
  [create-joint-M
   ask randomapplicator [create-EM]
   pay-cooperative-marketing-cost-1
   calculate-SMCV
   ask randomapplicator [calculate-ECV]
   compare-c-customer-value]
  
  [set acceptance? 0
   pay-fixed-cost 
   set suppliertick suppliertick + 1
   set implementation-list lput 0 implementation-list
   do-1st-cooperative-output
   set applicator-preference -1]
end 


to pay-fixed-cost
  set capital capital - 2
  if capital < 2 [set capital 0]  
  
  set costs costs + 2
  set sumcosts sumcosts + 2
end


to create-joint-M
  let smc0k item 0 smc
  let k0 item smc0k sknowledge-fields
  let smc1k item 1 smc
  let k1 item smc1k sknowledge-fields
  let smc0a item 0 smc
  let a0 item smc0a samounts
  let smc1a item 1 smc
  let a1 item smc1a samounts
 
  ask randommanufacturer                                                       ; manufacturer tries to improve the supplier's marketing concept by searching  
  [                                                                            ; for the same knowledge fields but with a higher amount value
   let i 0
   while [i < length mknowledge-fields]
   [let kvalue item i mknowledge-fields
    let avalue item i mamounts 
    let evalue item i mexpertises
    if kvalue = k0 and avalue > a0 
    [ask randomsupplier [set samounts replace-item smc0a samounts avalue
     set sexpertises replace-item smc0a sexpertises evalue]]
    if kvalue = k1 and avalue > a1 
    [ask randomsupplier [set samounts replace-item smc1a samounts avalue
     set sexpertises replace-item smc1a sexpertises evalue]]
    set i i + 1
   ] 
  ]

  set smc0k item 0 smc
  set k0 item smc0k sknowledge-fields
  set smc1k item 1 smc
  set k1 item smc1k sknowledge-fields
  set smc0a item 0 smc
  set a0 item smc0a samounts
  set smc1a item 1 smc
  set a1 item smc1a samounts
  set joint-M (k0 * a0) + (k1 * a1)                                            ; calculation of joint message
end

to pay-cooperative-marketing-cost-1                                            ; calculate costs for first cooperative attempt
  let difference abs (joint-M - EM)
  set cost abs (((15 / 30) * difference) + 1)
  set capital (capital - cost)
    
  set costs costs + cost
  set sumcosts sumcosts + cost 
   
  if capital < 2 
  [ask randomsupplier [set capital 0]]
end


to calculate-SMCV
  let smc0k item 0 smc
  let k0 item smc0k sknowledge-fields
  let a0 item smc0k samounts
  let a0double a0
  ifelse a0 < 2 [set a0 0][set a0 1]
  let smc1k item 1 smc
  let k1 item smc1k sknowledge-fields
  let a1 item smc1k samounts
  let a1double a1
  ifelse a1 < 2 [set a1 0][set a1 1]
  
  let smc0e item 0 smc
  let e0 item smc0e sexpertises
  let smc1e item 1 smc
  let e1 item smc1e sexpertises 
  
  if k0 >= 3 and k0 <= 4 and k1 >= 5 and k1 <= 6 
  [if a0double = 3 [set k0 6]]
  
  
  if k1 >= 3 and k1 <= 4 and k0 >= 5 and k0 <= 6 
  [if a1double = 3 [set k1 6]]
  
  let list2 []
  set list2 lput k0 list2 
  set list2 lput k1 list2 
   
  set SMCV (a0 *(k0 * e0)) + (a1 *(k1 * e1))                                   ; calculation of customer value of supplier and manufacturer 
end


to compare-c-customer-value                                                    ; compare cooperative customer value
  let knowmanufacturer []
  let knowapplicator []
  let knowsupplier []
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  ask randommanufacturer
  [set knowmanufacturer mknowledge-fields]
   
  let combined-sm-knowledgefields sentence sknowledge-fields knowmanufacturer                              ; pooling knowledge fields of supplier and manufacturer
  set combined-sm-knowledgefields remove-duplicates combined-sm-knowledgefields
  set combined-sm-knowledgefields sort combined-sm-knowledgefields  
 
  set c-knowledge-overlap 0
  
  let i 0
  while                                                                                                    ; calculate cooperative knowledge overlap
  [i < length combined-sm-knowledgefields]
  [let svalue item i combined-sm-knowledgefields
   if member? svalue knowapplicator[set c-knowledge-overlap c-knowledge-overlap + 1]
   set i i + 1
  ]
 
  if c-knowledge-overlap = 0 [set s-m-customer-value-tolerance 1.00]                                       ; not used at the moment
  if c-knowledge-overlap > 0 and c-knowledge-overlap <= 2 [set s-m-customer-value-tolerance 1.00]          ; not used at the moment
  if c-knowledge-overlap > 2 and c-knowledge-overlap <= 4 [set s-m-customer-value-tolerance 1.00]          ; not used at the moment
  if c-knowledge-overlap > 4 [set s-m-customer-value-tolerance 1.00]                                       ; not used at the moment
 
  ifelse SMCV * s-m-customer-value-tolerance >= ECV   
  [ask randomapplicator
    [
     ifelse selection-progress <= 3 and SMCV > customer-value-memory 
     [
      set customer-value-memory SMCV 
      set selection-progress selection-progress + 1
 
      if past-supplier-strategy = "cooperative"                                                            ; due to the higher customer value of current supplier, the applicator
      [ask supplier cooperative-supplier-link-memory [set succeed? false set implemented? 0]               ; rejects the former supplier's attempt and changes related attributes
       
       set i 0
       while [i < length implementation-list]
       [let value item i implementation-list
        let applicator item i applicator-implementation-list
        if value = 1 and applicator = [who] of randomapplicator 
        [set implementation-list replace-item i implementation-list 0]
        set i i + 1] 
      ]
            
      if past-supplier-strategy = "non-cooperative" 
      [ask supplier non-cooperative-supplier-link-memory [set succeed? false set implemented? 0]] 
       
      let j 0
       while [j < length implementation-list]
       [let value item j implementation-list
        let applicator item j applicator-implementation-list
        if value = 1 and applicator = [who] of randomapplicator 
        [set implementation-list replace-item j implementation-list 0]
        set j j + 1] 
      
      set cooperative-manufacturer-link-memory [who] of randommanufacturer
      set cooperative-supplier-link-memory [who] of randomsupplier
     
      set implementation-list lput 1 implementation-list
      
      set selection-progress selection-progress + 1
      set past-supplier-strategy "cooperative"
      
      ask randomsupplier 
      [set implemented? 1
       set suppliertick suppliertick + 1
       set acceptance? 1
       set applicator-preference [who] of randomapplicator
       pay-cooperative-marketing-cost-2
       do-2nd-cooperative-output
       set manufacturer-preference [who] of randommanufacturer
       set supplier-manufacturer-memory [who] of randommanufacturer
       set supplier-applicator-memory [who] of randomapplicator
       set succeed? true] 
     ]
     
     [ask randomsupplier
       [set acceptance? 0
        set suppliertick suppliertick + 1
        pay-cooperative-marketing-cost-2
        set implementation-list lput 0 implementation-list
        do-2nd-cooperative-output
        set applicator-preference -1
        set manufacturer-preference -1]]  
    ]
  ]
  
  [set suppliertick suppliertick + 1
   set acceptance? 0
   pay-cooperative-marketing-cost-2
   set implementation-list lput 0 implementation-list
   do-2nd-cooperative-output
   start-next-round 
   set applicator-preference -1
   set manufacturer-preference -1]
end


to pay-cooperative-marketing-cost-2                                                                         ; calculate costs for second cooperative attempt
  let knowmanufacturer [] 
  let knowapplicator []
  let knowsupplier []
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  ask randommanufacturer
  [set knowmanufacturer mknowledge-fields]
  
  let combined-sm-knowledgefields sentence sknowledge-fields knowmanufacturer
  set combined-sm-knowledgefields remove-duplicates combined-sm-knowledgefields
  set combined-sm-knowledgefields sort combined-sm-knowledgefields  
 
  set c-knowledge-overlap 0
  
  let i 0
  while 
  [i < length combined-sm-knowledgefields]
  [let svalue item i combined-sm-knowledgefields
   if member? svalue knowapplicator[set c-knowledge-overlap c-knowledge-overlap + 1]
   set i i + 1
  ] 
  
  if c-knowledge-overlap = 0 [set c-costfactor 1.00]                                                        ; not used at the moment
  if c-knowledge-overlap > 0 and c-knowledge-overlap <= 2 [set c-costfactor 1.00]                           ; not used at the moment
  if c-knowledge-overlap > 2 and c-knowledge-overlap <= 4 [set c-costfactor 1.00]                           ; not used at the moment
  if c-knowledge-overlap > 4 [set c-costfactor 1.00]                                                        ; not used at the moment
  
  let difference abs (SMCV - ECV)
  set cost abs (((15 / 36) * difference + 1) * c-costfactor)
  set capital (capital - cost)
    
  set costs costs + cost
  set sumcosts sumcosts + cost 
   
  if capital < 2 
  [ask randomsupplier [set capital 0]]
end


to start-next-attempt                                                                                       ; supplier tries to improve its marketing concept by searching for triples
  let firstvalue-position item 0 smc                                                                        ; with the same knowledge fields but with a higher amount value 
  let firstvalue item firstvalue-position sknowledge-fields
  let firstamount item firstvalue-position samounts
  let sknowledge-fields-double sknowledge-fields
  set sknowledge-fields-double replace-item firstvalue-position sknowledge-fields-double 0

  let smc1 smc

  let i 0
  while [i < length sknowledge-fields]
  [
   let value item i sknowledge-fields
   let amount item i samounts
   if value = firstvalue and amount > firstamount [set smc replace-item 0 smc i]  
   set i i + 1
  ]

  let secondvalue-position item 1 smc
  let secondvalue item secondvalue-position sknowledge-fields 
  let secondamount item secondvalue-position samounts
  set sknowledge-fields-double replace-item secondvalue-position sknowledge-fields-double 0

  let smc2 smc

  if smc2 = smc1 
  [let i2 0
   while [i2 < length sknowledge-fields]
   [
    let value2 item i2 sknowledge-fields
    let amount2 item i2 samounts
    if value2 = secondvalue and amount2 > secondamount [set smc replace-item 1 smc i2]  
    set i2 i2 + 1
   ]
  ]  
end


to start-next-round
  adjust-amounts-of-supplier
  adjust-expertises-of-supplier
end

to adjust-amounts-of-supplier                                                                              ; increase the current amount values by 1
  let location 0
  while [location < length sknowledge-fields] 
  [let amount item location samounts
    ifelse member? location smc
    [if amount < 3 [set samounts replace-item location samounts (amount + 1)]]
  []
  set location location + 1
  ]
end

to adjust-expertises-of-supplier                                                                           ; increase the current expertise values by 1
  let location 0
  while [location < length sknowledge-fields] 
  [let expertise item location sexpertises
   ifelse member? location smc
   [if expertise < 3 [set sexpertises replace-item location sexpertises (expertise + 1)]]
   []
   set location location + 1
  ]
end


to setup-output                                                                                            ; create 3 different outputs
  let filename (word "output-results.txt")
  file-open filename
  let mylist []
  set mylist ["S-Identity"";" "VCM"";" "R"";" "SMC-Triples"";" "SK1"";" "SK2"";" "DoI"";" "A-Identity"";" "AMC-Triples"";" "AK1"";" "AK2"";" "exp-DoI"";" "Sender-Ks"";" "Receiver-Ks"";" "KO"";" "M-Identity"";" "SCV"";" "nc-ECV"";" "SMCV"";" "c-ECV"";" "Attempt"";" "Accept"";" "Cost per Step"";" "Sum Costs"";" "Accept + Implement"]
  file-print mylist
  file-close-all
  
  let filename3 (word "last-row-output-results.txt")
  file-open filename3
  let mylist3 []
  set mylist ["S-Identity"";" "VCM"";" "R"";" "SMC-Triples"";" "SK1"";" "SK2"";" "DoI"";" "A-Identity"";" "AMC-Triples"";" "AK1"";" "AK2"";" "exp-DoI"";" "Sender-Ks"";" "Receiver-Ks"";" "KO"";" "M-Identity"";" "SCV"";" "nc-ECV"";" "SMCV"";" "c-ECV"";" "Attempt"";" "Accept"";" "Cost per Step"";" "Sum Costs"";" "Accept + Implement"]
  file-print mylist
  file-close-all
  
  let filename2 (word "implementation-output-results.txt")
  file-open filename2
  let mylist2 []
  set mylist ["S-Identity"";" "A-Identity"";" "Accept + Implement"]
  file-print mylist
  file-close-all 
end


to do-1st-non-cooperative-output
  set attributes []
  let filename (word "output-results.txt")
  file-open filename

  set attributes lput (word [who] of randomsupplier ";") attributes
  set supplier-implementation-list lput [who] of randomsupplier supplier-implementation-list
  set attributes lput (word "1" ";") attributes
  set attributes lput (word innovation-rate ";") attributes
  
  let triple1 []
  let smc0 item 0 smc
  let k0 item smc0 sknowledge-fields
  set triple1 lput k0 triple1
  let a0 item smc0 samounts
  set triple1 lput a0 triple1
  let e0 item smc0 sexpertises
  set triple1 lput e0 triple1
  
  let triple2 []
  let smc1 item 1 smc
  let k1 item smc1 sknowledge-fields
  set triple2 lput k1 triple2
  let a1 item smc1 samounts
  set triple2 lput a1 triple2
  let e1 item smc1 sexpertises
  set triple2 lput e1 triple2

  set attributes lput (word  triple1 triple2 ";") attributes
  
  set attributes lput (word k0 ";") attributes 
  set attributes lput (word k1 ";") attributes

  if (k0 >= 3 and k0 <= 4) and (k1 >= 3 and k1 <= 4) [set attributes lput (word "0" ";") attributes]
  if (k0 >= 3 and k0 <= 4 and k1 >= 5) or (k0 >= 5 and k1 >= 3 and k1 <= 4) [set attributes lput (word "2" ";") attributes]
  if (k0 >= 5) and (k1 >= 5) [set attributes lput (word "1" ";") attributes]
  
  ask randomsupplier [set attributes lput (word [who] of randomapplicator ";") attributes
  set applicator-implementation-list lput [who] of randomapplicator applicator-implementation-list]
  
  ask randomapplicator 
  [
   let atriple1 []  
   let amc0 item 0 amc
   let ak0 item amc0 aknowledge-fields
   set atriple1 lput ak0 atriple1
   let aa0 item amc0 aamounts
   set atriple1 lput aa0 atriple1
   let ae0 item amc0 aexpertises
   set atriple1 lput ae0 atriple1
  
   let atriple2 []
   let amc1 item 1 amc
   let ak1 item amc1 aknowledge-fields
   set atriple2 lput ak1 atriple2
   let aa1 item amc1 aamounts
   set atriple2 lput aa1 atriple2
   let ae1 item amc1 aexpertises
   set atriple2 lput ae1 atriple2
  
   ask randomsupplier [set attributes lput (word atriple1 atriple2 ";") attributes
   
   set attributes lput (word ak0 ";") attributes
   set attributes lput (word ak1 ";") attributes
  
   if (ak0 >= 3 and ak0 <= 4) and (ak1 >= 3 and ak1 <= 4) [set attributes lput (word "0" ";") attributes]
   if (ak0 >= 3 and ak0 <= 4 and ak1 >= 5) or (ak0 >= 5 and ak1 >= 3 and ak1 <= 4) [set attributes lput (word "2" ";") attributes]
   if (ak0 >= 5) and (ak1 >= 5) [set attributes lput (word "1" ";") attributes]]
  ]
  
  let knowapplicator []
  let knowsupplier []
  set knowsupplier remove-duplicates sknowledge-fields
  set knowsupplier sort knowsupplier
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  set nc-knowledge-overlap 0
  
  let i 0
  while 
  [i < length knowsupplier]
  [let svalue item i knowsupplier
   if member? svalue knowapplicator[set nc-knowledge-overlap nc-knowledge-overlap + 1]
   set i i + 1
  ]
  set attributes lput (word knowsupplier ";") attributes
  set attributes lput (word knowapplicator ";") attributes
  set attributes lput (word nc-knowledge-overlap ";") attributes 
 
  set attributes lput (word [who] of randommanufacturer ";") attributes

  set attributes lput (word "-" ";" ) attributes 
  set attributes lput (word "-" ";") attributes
  
  set attributes lput (word "-" ";") attributes
  set attributes lput (word "-" ";") attributes
  
  set attributes lput (word suppliertick ";") attributes
  set attributes lput (word acceptance? ";") attributes
  set attributes lput (word costs ";") attributes
  set attributes lput (word sumcosts ";") attributes
  set attributes lput (word implemented? ";") attributes
  
  file-print attributes
  file-close-all
end


to do-2nd-non-cooperative-output
  set attributes []
  let filename (word "output-results.txt")
  file-open filename
 
  set attributes lput (word [who] of randomsupplier ";") attributes
  set supplier-implementation-list lput [who] of randomsupplier supplier-implementation-list
  set attributes lput (word "1" ";") attributes
  set attributes lput (word innovation-rate ";") attributes
   
  let triple1 []
  let smc0 item 0 smc
  let k0 item smc0 sknowledge-fields
  set triple1 lput k0 triple1
  let a0 item smc0 samounts
  set triple1 lput a0 triple1
  let e0 item smc0 sexpertises
  set triple1 lput e0 triple1
  
  let triple2 []
  let smc1 item 1 smc
  let k1 item smc1 sknowledge-fields
  set triple2 lput k1 triple2
  let a1 item smc1 samounts
  set triple2 lput a1 triple2
  let e1 item smc1 sexpertises
  set triple2 lput e1 triple2

  set attributes lput (word  triple1 triple2 ";") attributes
  
  set attributes lput (word k0 ";") attributes 
  set attributes lput (word k1 ";") attributes

  if (k0 >= 3 and k0 <= 4) and (k1 >= 3 and k1 <= 4) [set attributes lput (word "0" ";") attributes]
  if (k0 >= 3 and k0 <= 4 and k1 >= 5) or (k0 >= 5 and k1 >= 3 and k1 <= 4) [set attributes lput (word "2" ";") attributes]
  if (k0 >= 5) and (k1 >= 5) [set attributes lput (word "1" ";") attributes]
  
  ask randomsupplier [set attributes lput (word [who] of randomapplicator ";") attributes
  set applicator-implementation-list lput [who] of randomapplicator applicator-implementation-list]
  
  ask randomapplicator 
  [
   let atriple1 []  
   let amc0 item 0 amc
   let ak0 item amc0 aknowledge-fields
   set atriple1 lput ak0 atriple1
   let aa0 item amc0 aamounts
   set atriple1 lput aa0 atriple1
   let ae0 item amc0 aexpertises
   set atriple1 lput ae0 atriple1
  
   let atriple2 []
   let amc1 item 1 amc
   let ak1 item amc1 aknowledge-fields
   set atriple2 lput ak1 atriple2
   let aa1 item amc1 aamounts
   set atriple2 lput aa1 atriple2
   let ae1 item amc1 aexpertises
   set atriple2 lput ae1 atriple2
  
   ask randomsupplier 
   [
    set attributes lput (word atriple1 atriple2 ";") attributes
    set attributes lput (word ak0 ";") attributes
    set attributes lput (word ak1 ";") attributes
  
    if (ak0 >= 3 and ak0 <= 4) and (ak1 >= 3 and ak1 <= 4) [set attributes lput (word "0" ";") attributes]
    if (ak0 >= 3 and ak0 <= 4 and ak1 >= 5) or (ak0 >= 5 and ak1 >= 3 and ak1 <= 4) [set attributes lput (word "2" ";") attributes]
    if (ak0 >= 5) and (ak1 >= 5) [set attributes lput (word "1" ";") attributes]
   ]
  ]
   
  let knowapplicator []
  let knowsupplier []
  set knowsupplier remove-duplicates sknowledge-fields
  set knowsupplier sort knowsupplier
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  set nc-knowledge-overlap 0
 
  let i 0
  while 
  [i < length knowsupplier]
  [let svalue item i knowsupplier
   if member? svalue knowapplicator[set nc-knowledge-overlap nc-knowledge-overlap + 1]
   set i i + 1
  ]
  set attributes lput (word knowsupplier ";") attributes
  set attributes lput (word knowapplicator ";") attributes
  set attributes lput (word nc-knowledge-overlap ";") attributes 
  
  set attributes lput (word [who] of randommanufacturer ";") attributes
  
  set attributes lput (word (SCV * s-customer-value-tolerance) ";") attributes 
  set attributes lput (word (ECV) ";") attributes
  
  set attributes lput (word "-" ";") attributes
  set attributes lput (word "-" ";") attributes
  
  set attributes lput (word suppliertick ";") attributes
  set attributes lput (word acceptance? ";") attributes
  set attributes lput (word costs ";") attributes
  set attributes lput (word sumcosts ";") attributes
  set attributes lput (word implemented? ";") attributes
  
  file-print attributes
  file-close-all
end


to do-1st-cooperative-output
  set attributes []
  let filename (word "output-results.txt")
  file-open filename
  
  set attributes lput (word [who] of randomsupplier ";") attributes
  set supplier-implementation-list lput [who] of randomsupplier supplier-implementation-list
  set attributes lput (word "0" ";") attributes
  set attributes lput (word innovation-rate ";") attributes
   
  let triple1 []
  let smc0 item 0 smc
  let k0 item smc0 sknowledge-fields
  set triple1 lput k0 triple1
  let a0 item smc0 samounts
  set triple1 lput a0 triple1
  let e0 item smc0 sexpertises
  set triple1 lput e0 triple1
  
  let triple2 []
  let smc1 item 1 smc
  let k1 item smc1 sknowledge-fields
  set triple2 lput k1 triple2
  let a1 item smc1 samounts
  set triple2 lput a1 triple2
  let e1 item smc1 sexpertises
  set triple2 lput e1 triple2

  set attributes lput (word  triple1 triple2 ";") attributes
  
  set attributes lput (word k0 ";") attributes 
  set attributes lput (word k1 ";") attributes

  if (k0 >= 3 and k0 <= 4) and (k1 >= 3 and k1 <= 4) [set attributes lput (word "0" ";") attributes]
  if (k0 >= 3 and k0 <= 4 and k1 >= 5) or (k0 >= 5 and k1 >= 3 and k1 <= 4) [set attributes lput (word "2" ";") attributes]
  if (k0 >= 5) and (k1 >= 5) [set attributes lput (word "1" ";") attributes]
  
  ask randomsupplier [set attributes lput (word [who] of randomapplicator ";") attributes
  set applicator-implementation-list lput [who] of randomapplicator applicator-implementation-list]
  
  ask randomapplicator 
  [
   let atriple1 []  
   let amc0 item 0 amc
   let ak0 item amc0 aknowledge-fields
   set atriple1 lput ak0 atriple1
   let aa0 item amc0 aamounts
   set atriple1 lput aa0 atriple1
   let ae0 item amc0 aexpertises
   set atriple1 lput ae0 atriple1
  
   let atriple2 []
   let amc1 item 1 amc
   let ak1 item amc1 aknowledge-fields
   set atriple2 lput ak1 atriple2
   let aa1 item amc1 aamounts
   set atriple2 lput aa1 atriple2
   let ae1 item amc1 aexpertises
   set atriple2 lput ae1 atriple2
  
   ask randomsupplier 
   [
    set attributes lput (word atriple1 atriple2 ";") attributes
    set attributes lput (word ak0 ";") attributes
    set attributes lput (word ak1 ";") attributes
  
    if (ak0 >= 3 and ak0 <= 4) and (ak1 >= 3 and ak1 <= 4) [set attributes lput (word "0" ";") attributes]
    if (ak0 >= 3 and ak0 <= 4 and ak1 >= 5) or (ak0 >= 5 and ak1 >= 3 and ak1 <= 4) [set attributes lput (word "2" ";") attributes]
    if (ak0 >= 5) and (ak1 >= 5) [set attributes lput (word "1" ";") attributes]
   ]
  ]
  
  let knowmanufacturer []
  let knowapplicator []
  let knowsupplier []
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  ask randommanufacturer 
  [set knowmanufacturer mknowledge-fields]
  
  let combined-sm-knowledgefields sentence sknowledge-fields knowmanufacturer
  set combined-sm-knowledgefields remove-duplicates combined-sm-knowledgefields
  set combined-sm-knowledgefields sort combined-sm-knowledgefields  
 
  set c-knowledge-overlap 0
 
  let i 0
  while 
  [i < length combined-sm-knowledgefields]
  [let svalue item i combined-sm-knowledgefields
   if member? svalue knowapplicator[set c-knowledge-overlap c-knowledge-overlap + 1]
   set i i + 1
  ] 
  set attributes lput (word combined-sm-knowledgefields ";") attributes
  set attributes lput (word knowapplicator ";") attributes
  set attributes lput (word c-knowledge-overlap ";") attributes 
 
  set attributes lput (word [who] of randommanufacturer ";") attributes
 
  set attributes lput (word "-" ";") attributes 
  set attributes lput (word "-" ";") attributes
  
  set attributes lput (word "-" ";") attributes 
  set attributes lput (word "-" ";") attributes  
  
  set attributes lput (word suppliertick ";") attributes
  set attributes lput (word acceptance? ";") attributes
  set attributes lput (word costs ";") attributes
  set attributes lput (word sumcosts ";") attributes
  set attributes lput (word implemented? ";") attributes
  
  file-print attributes
  file-close-all
end


to do-2nd-cooperative-output
  set attributes []
  let filename (word "output-results.txt")
  file-open filename
  
  set attributes lput (word [who] of randomsupplier ";") attributes
  set supplier-implementation-list lput [who] of randomsupplier supplier-implementation-list
  set attributes lput (word "0" ";") attributes
  set attributes lput (word innovation-rate ";") attributes
   
  let triple1 []
  let smc0 item 0 smc
  let k0 item smc0 sknowledge-fields
  set triple1 lput k0 triple1
  let a0 item smc0 samounts
  set triple1 lput a0 triple1
  let e0 item smc0 sexpertises
  set triple1 lput e0 triple1
  
  let triple2 []
  let smc1 item 1 smc
  let k1 item smc1 sknowledge-fields
  set triple2 lput k1 triple2
  let a1 item smc1 samounts
  set triple2 lput a1 triple2
  let e1 item smc1 sexpertises
  set triple2 lput e1 triple2

  set attributes lput (word  triple1 triple2 ";") attributes
  
  set attributes lput (word k0 ";") attributes 
  set attributes lput (word k1 ";") attributes

  if (k0 >= 3 and k0 <= 4) and (k1 >= 3 and k1 <= 4) [set attributes lput (word "0" ";") attributes]
  if (k0 >= 3 and k0 <= 4 and k1 >= 5) or (k0 >= 5 and k1 >= 3 and k1 <= 4) [set attributes lput (word "2" ";") attributes]
  if (k0 >= 5) and (k1 >= 5) [set attributes lput (word "1" ";") attributes]
  
  ask randomsupplier [set attributes lput (word [who] of randomapplicator ";") attributes
  set applicator-implementation-list lput [who] of randomapplicator applicator-implementation-list]
  
  ask randomapplicator 
  [
   let atriple1 []  
   let amc0 item 0 amc
   let ak0 item amc0 aknowledge-fields
   set atriple1 lput ak0 atriple1
   let aa0 item amc0 aamounts
   set atriple1 lput aa0 atriple1
   let ae0 item amc0 aexpertises
   set atriple1 lput ae0 atriple1
  
   let atriple2 []
   let amc1 item 1 amc
   let ak1 item amc1 aknowledge-fields
   set atriple2 lput ak1 atriple2
   let aa1 item amc1 aamounts
   set atriple2 lput aa1 atriple2
   let ae1 item amc1 aexpertises
   set atriple2 lput ae1 atriple2
  
   ask randomsupplier 
   [
    set attributes lput (word atriple1 atriple2 ";") attributes
    set attributes lput (word ak0 ";") attributes
    set attributes lput (word ak1 ";") attributes
  
    if (ak0 >= 3 and ak0 <= 4) and (ak1 >= 3 and ak1 <= 4) [set attributes lput (word "0" ";") attributes]
    if (ak0 >= 3 and ak0 <= 4 and ak1 >= 5) or (ak0 >= 5 and ak1 >= 3 and ak1 <= 4) [set attributes lput (word "2" ";") attributes]
    if (ak0 >= 5) and (ak1 >= 5) [set attributes lput (word "1" ";") attributes]
   ]
  ]
   
  let knowmanufacturer []
  let knowapplicator []
  let knowsupplier []
  
  ask randomapplicator
  [set knowapplicator remove-duplicates aknowledge-fields
   set knowapplicator sort knowapplicator] 
  
  ask randommanufacturer 
  [set knowmanufacturer mknowledge-fields]
  
  let combined-sm-knowledgefields sentence sknowledge-fields knowmanufacturer
  set combined-sm-knowledgefields remove-duplicates combined-sm-knowledgefields
  set combined-sm-knowledgefields sort combined-sm-knowledgefields  
 
  set c-knowledge-overlap 0
  
  let i 0
  while 
  [i < length combined-sm-knowledgefields]
  [let svalue item i combined-sm-knowledgefields
   if member? svalue knowapplicator[set c-knowledge-overlap c-knowledge-overlap + 1]
   set i i + 1
  ]
  set attributes lput (word combined-sm-knowledgefields ";") attributes
  set attributes lput (word knowapplicator ";") attributes
  set attributes lput (word c-knowledge-overlap ";") attributes 
  
  set attributes lput (word [who] of randommanufacturer ";") attributes
 
  set attributes lput (word "-" ";") attributes 
  set attributes lput (word "-" ";") attributes
  
  set attributes lput (word (SMCV * s-m-customer-value-tolerance) ";") attributes
  set attributes lput (word ECV ";") attributes
  
  set attributes lput (word suppliertick ";") attributes
  set attributes lput (word acceptance? ";") attributes
  set attributes lput (word costs ";") attributes
  set attributes lput (word sumcosts ";") attributes
  set attributes lput (word implemented? ";") attributes
  
  file-print attributes
  file-close-all
end


to plotimplementation
  let i 0
  while [i < length implementation-list]
  [let filename2 (word "implementation-output-results.txt")
   file-open filename2
   let attributes2 []
   let supplier item i supplier-implementation-list
   let applicator item i applicator-implementation-list
   let implemented item i implementation-list
   set attributes2 lput (word supplier ";") attributes2
   set attributes2 lput (word applicator ";") attributes2
   set attributes2 lput (word implemented ";") attributes2
   file-print attributes2
   file-close-all
   set i i + 1]   
end


to plotoutput
  let filename (word "last-row-output-results.txt")
  file-open filename
    
  ask suppliers with [active? = false] [die]
  
  file-print attributes
  file-close-all
end
@#$#@#$#@
GRAPHICS-WINDOW
264
55
509
206
10
7
8.0
1
8
1
1
1
0
0
0
1
-10
10
-7
7
1
1
1
ticks
1.0

INPUTBOX
181
55
259
115
initial-capital
15
1
0
Number

BUTTON
5
16
91
49
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
95
16
175
49
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
5
138
176
171
number-of-suppliers
number-of-suppliers
0
100
50
5
1
NIL
HORIZONTAL

SLIDER
6
212
175
245
number-of-manufacturers
number-of-manufacturers
0
100
40
5
1
NIL
HORIZONTAL

SLIDER
6
250
175
283
number-of-applicators
number-of-applicators
0
100
30
5
1
NIL
HORIZONTAL

SLIDER
6
173
176
206
big-supplier's-ratio
big-supplier's-ratio
0.00
1.00
0.1
0.05
1
NIL
HORIZONTAL

SLIDER
5
103
175
136
innovation-rate
innovation-rate
0
0.50
0.4
0.05
1
NIL
HORIZONTAL

CHOOSER
5
54
175
99
strategy
strategy
"cooperative" "non-cooperative" "random strategy"
2

INPUTBOX
181
118
260
178
trials-number
0
1
0
Number

@#$#@#$#@
## WHAT IS IT?

Inspired by the SKIN model, the basic concept here is to model the acceptance and implementation of supplier innovations. This model includes three types of agents comprising suppliers, manufacturers and applicators. 

The agents represent suppliers try to implement their innovations via VCM and thereby have to target applicators. In this model, the supplier firm chooses between a cooperative VCM strategy, whereas the supplier and the manufacturer firm together present their innovation to the applicator and a non-cooperative VCM strategy, whereas the supplier firm tries to implement its innovation by its own. The success of the marketing attempt is measured in terms of the acceptance and implementation of the innovation as well as the marketing resources used. The basic model is extended with a representation of knowledge dynamics in and between firms. In particular, the supplier firm attempts to increase its marketing success by improving its knowledge base through adaptation to applicators’ needs and individual learning. 


## HOW IT WORKS

At the beginning of each simulation experiment, the first step is the initialization of the agents’ kenes by executing the setup button. A kene consists of a vector containing different units of knowledge called triples. Each triple incorporates the field of knowledge (K) in which a firm operates, the amount of knowledge (A) a firm has in a specific field, and the expertise level (E) which reflects a firm’s experience gathered in communication. During the initialization, the marketing hypotheses of the agents are also built from their kene sets. This hypothesis describes the source a firm uses for its attempts to make profits on the market. It is composed by two triples out of the kene set. These triples represent different teams or persons in a firm who are working together to address potential business partners. 

After the initialization, all agents are created and ready to play their roles. The VCM process is started by executing the go button. A single process ends if there is the supplier innovation is implemented or rejected. If an innovation is rejected, the supplier firm tries to improve its marketing hypothesis and start a new attempt. If possible, it searches through the qualified triples and selects a triple with a higher amount of knowledge. In addition, the experience level increases by one because the supplier has obtained learning experience. The supplier has to pay for each step within the marketing process. Consequently, the simulation stops if the supplier firm has implemented its innovation successfully or has no budget left to promote its innovation again. The users of the model are given the options to run the process using cooperative VCM or non-cooperative VCM. The two processes are separated to enable the comparison of their effectiveness and efficiency. Furthermore, there is the opportunity to use the random function, so that the suppliers are choosing their marketing strategy on their own. 


## HOW TO USE IT


- The initial-capital input defines the starting capital for all suppliers.
- The number-of-suppliers slider defines the numbers of suppliers.
- With the innovation-rate slider the user defines the percentage of how many suppliers   within the run are actually active. 
- The big-supplier´s-ratio is used to define a percentage of suppliers who are getting    additional capital to promote their innovations. 
- The number-of-manufacturers slider defines the numbers of manufacturers.
- The number-of-applicators slider defines the numbers of applicators.     
- The strategy-chooser is used to choose between a cooperative, a non-cooperative or a    random strategy. Within the random strategy the supplier chooses between a              cooperative and non-cooperative strategy randomly.
- Pressing the setup button starts the initialization of the agents. Once the             simulation has been setup, you are now ready to run it, by pushing the GO button. 


## THINGS TO NOTICE

The VCM model creates a .txt document with all the values of all active suppliers for each tick. The .txt document will be saved at the same path the VCM model is saved.              

## RELATED MODELS

Simulating knowledge dynamics in innovation networks (SKIN) developed by Petra Ahrweiler, Nigel Gilbert and Andreas Pyka

## CREDITS AND REFERENCES

Ahrweiler, P., Pyka, A. and Gilbert, N. (2004). Simulating knowledge dynamics in innovation networks (SKIN). Volkswirtschaftliche Diskussionsreihe 267, Institute of Economics, Uni-versity of Augsburg

Ahrweiler, P., Pyka, A. and Gilbert, N. (2011). A new model for university links in knowledge-based economies. Journal of Product Innovation Management 28(2), 218-235

Garcia, R. and Jager, W. (2011). Agent-based modeling of innovation diffusion. Journal of Product Innovation Management 28(2), 148-151

Gilbert, N. (1997). A simulation of the structure of academic science. Sociological Research Online 2(2)

Gilbert, N., Pyka, A. and Ahrweiler, P. (2001). Innovation networks – A simulation approach. Journal of Artificial Societies and Social Simulation 4(3)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

factory
false
0
Rectangle -7500403 true true 76 194 285 270
Rectangle -7500403 true true 36 95 59 231
Rectangle -16777216 true false 90 210 270 240
Line -7500403 true 90 195 90 255
Line -7500403 true 120 195 120 255
Line -7500403 true 150 195 150 240
Line -7500403 true 180 195 180 255
Line -7500403 true 210 210 210 240
Line -7500403 true 240 210 240 240
Line -7500403 true 90 225 270 225
Circle -1 true false 37 73 32
Circle -1 true false 55 38 54
Circle -1 true false 96 21 42
Circle -1 true false 105 40 32
Circle -1 true false 129 19 42
Rectangle -7500403 true true 14 228 78 270

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
